import os
import json
import time
import requests
import traceback

import vsource_algorithm.exceptions as exceptions
from vsource_algorithm.loginInstance import login_instance
import vsource_algorithm.configs as configs

def face_attribute(face_path, max_interval=configs.max_interval, version='none'):
    face_upload_url = configs.deepface_upload_url
    face_submit_url = configs.deepface_submit_url
    face_result_url = configs.deepface_result_url
    if version == 'fsx':
        face_upload_url = configs.deepface3_upload_url
        face_submit_url = configs.deepface3_submit_url
        face_result_url = configs.deepface3_result_url
    if version == 'cgy-2':
        face_upload_url = configs.deepface2_upload_url
        face_submit_url = configs.deepface2_submit_url
        face_result_url = configs.deepface2_result_url
    headers = {'Cookie': login_instance.cookie}
    with open(face_path, 'rb') as f1:
        filename1 = os.path.split(face_path)[-1]
        ext1 = os.path.splitext(face_path)[-1]
        if ext1 not in ['.jpg', '.png', '.jpeg']:
            raise exceptions.FileExtNotValid('[VSOURCE-Lib] 图像扩展名不对，请检查是否是png或者jpg')
        if ext1 == '.jpg':
            filetype = 'image/jpg'
        if ext1 == '.png':
            filetype = 'image/png'
        if ext1 == '.jpeg':
            filetype = 'image/jpeg'

        uploaded_response1 = requests.post(face_upload_url, files={
            'file':  (filename1, f1.read(), filetype)
        }, headers=headers)
        # print(uploaded_response1.content)
        if uploaded_response1.status_code == 403:
            raise exceptions.LoginError('[VSOURCE-Lib] 登陆信息失败，请先正常的登陆')

        uploaded_path1 = json.loads(uploaded_response1.content)['return_path']

    post_params = {
        'face_name': uploaded_path1
    }
    response = requests.post(face_submit_url, data=post_params, headers=headers)
    if response.status_code == 403:
        raise exceptions.LoginError('[VSOURCE-Lib] 登陆信息失败，请先正常的登陆')
    response_dict = json.loads(response.content)
    task_id = response_dict['id']
    start_time = time.time()
    while True:
        # 轮询查询结果
        if time.time() - start_time >= max_interval:
            raise exceptions.TimeOutException('[VSOURCE-Lib] 请求超时，未知错误')
        face_result = requests.get(face_result_url, params={'id': task_id}, headers=headers, timeout=max_interval)
        face_ans = json.loads(face_result.content)
        if face_ans['status'] != 200:
            time.sleep(configs.interval)
            continue
        try:
            assert face_ans['status'] == 200
            if face_ans['result']['status'] == 'error':
                return []
            return face_ans['result']['result']
        except Exception as e:
            return []

    raise exceptions.TimeOutException('[VSOURCE-Lib] 请求超时，未知错误')


# -*- coding: utf-8 -*-
# @Author   : Ecohnoch(xcy)
# @File     : __init__.py.py
# @Function : TODO

class FileExtNotValid(Exception):
    def __init__(self, err_msg):
        Exception.__init__(self, err_msg)

class TimeOutException(Exception):
    def __init__(self, err_msg):
        Exception.__init__(self, err_msg)

class LoginError(Exception):
    def __init__(self, err_msg):
        Exception.__init__(self, err_msg)


import os
import json
import time
import requests
import traceback

import vsource_algorithm.exceptions as exceptions
from vsource_algorithm.loginInstance import login_instance
import vsource_algorithm.configs as configs

def face_detection(face_path):
    face_detection_upload_url = configs.face_detection_upload_url
    face_detection_result_url = configs.face_detection_result_url
    headers = {'Cookie': login_instance.cookie}
    with open(face_path, 'rb') as f:
        filename = os.path.split(face_path)[-1]
        ext = os.path.splitext(face_path)[-1]
        if ext not in ['.jpg', '.png']:
            raise exceptions.FileExtNotValid('[VSOURCE-Lib] 图像扩展名不对，请检查是否是png或者jpg')
        if ext == '.jpg':
            filetype = 'image/jpg'
        if ext == '.png':
            filetype = 'image/png'
        uploaded_response = requests.post(face_detection_upload_url, files={
            'file':  (filename, f.read(), filetype)
        }, headers=headers)
        if uploaded_response.status_code == 403:
            raise exceptions.LoginError('[VSOURCE-Lib] 登陆信息失败，请先正常的登陆')
        print(uploaded_response.content)
        uploaded_path = json.loads(uploaded_response.content)['return_path']
    image_params = {
        'image_path': uploaded_path
    }
    response = requests.get(face_detection_result_url, params=image_params, headers=headers)
    if response.status_code == 403:
        raise exceptions.LoginError('[VSOURCE-Lib] 登陆信息失败，请先正常的登陆')
    response_dict = json.loads(response.content)
    return response_dict


# -*- coding: utf-8 -*-
# @Author   : Ecohnoch(xcy)
# @File     : configs.py
# @Function : TODO

interval = 0.3
max_interval = 10

login_url = 'http://120.26.143.61/login'
face_upload_url = 'http://120.26.143.61/face_upload'
face_submit_url = 'http://120.26.143.61/face_submit'
face_result_url = 'http://120.26.143.61/get_result'

speaker_upload_url = 'http://120.26.143.61/speaker_upload'
speaker_submit_url = 'http://120.26.143.61/speaker_submit'
speaker_result_url = 'http://120.26.143.61/speaker/get_result'

face_detection_upload_url = 'http://120.26.143.61/face_detection_upload'
face_detection_result_url = 'http://120.26.143.61/face_detection/get_result'

deepface_upload_url = 'http://120.26.143.61/deepface_upload'
deepface_submit_url = 'http://120.26.143.61/deepface_submit'
deepface_result_url = 'http://120.26.143.61/deepface/get_result'

deepface2_upload_url = 'http://120.26.143.61/deepface2_upload'
deepface2_submit_url = 'http://120.26.143.61/deepface2_submit'
deepface2_result_url = 'http://120.26.143.61/deepface2/get_result'

deepface3_upload_url = 'http://120.26.143.61/deepface3_upload'
deepface3_submit_url = 'http://120.26.143.61/deepface3_submit'
deepface3_result_url = 'http://120.26.143.61/deepface3/get_result'


# -*- coding: utf-8 -*-
# @Author   : Ecohnoch(xcy)
# @File     : loginInfoInsance.py
# @Function : TODO

import json
import requests
import traceback
import vsource_algorithm.configs as configs

class Singleton(type):
    _instances = {}

    def __call__(cls, *args, **kwargs):
        if cls not in cls._instances:
            cls._instances[cls] = super(Singleton, cls).__call__(*args, **kwargs)
        return cls._instances[cls]


class LoginInstance(metaclass=Singleton):
    def __init__(self, cookie=None):
        self.cookie = cookie

    def set_cookie(self, cookie):
        self.cookie = cookie

login_instance = LoginInstance()

def login(username, password):
    try:
        login_url = configs.login_url
        login_data = {
            'username': username,
            'password': password
        }
        response = requests.post(login_url, data=login_data)
        response_dict = json.loads(response.content)
        if response_dict['status'] == 200:
            login_instance.set_cookie(response.headers['Set-Cookie'])
            return True
        return False
    except Exception as e:
        traceback.print_exc()
        return False


import os
import inspect
import traceback

class AlgorithmContainerConfigParser:
    def __init__(self, config):
        self.cur_dir = os.path.dirname(__file__)

        self.logs = []
        self.pre_check(config)

    def pre_check(self, config):
        try:
            self.service_dir       = config['service-dir']
            self.service_filename  = config['service-filename']
            if str(self.service_filename).endswith('.py'):
                self.service_filename = self.service_filename[:-3]
            self.service_interface = config['service-interface']
            self.algorithm_name    = config['algorithm-name']
            self.requirements      = config['requirements']
            self.pre_command       = config['pre-command']
            self.input_params      = config['input-params']
            self.output_params     = config['output-params']
            assert os.path.exists(self.service_dir)
        except AssertionError:
            err_msg = traceback.format_exc()
            traceback.print_exc()
            self.logs.append('[Step 0 Error, Service Directory not exists]: ' + err_msg)
        except KeyError:
            err_msg = traceback.format_exc()
            traceback.print_exc()
            self.logs.append('[Step 0 Error, KeyError, invalid config]: ' + err_msg)

    # 第一步，生成requirements.txt
    def gen_requirements_txt(self):
        try:
            with open(os.path.join(self.service_dir, 'vsource_requirements.txt'), 'w') as f:
                for each_requirement in self.requirements:
                    f.write(each_requirement)
                    f.write('\n')
                f.write("redis\n")
                f.write('kafka-python\n')
            self.logs.append('[Step 1/6] Requirements has been successfully generated!')
        except Exception as e:
            err_msg = traceback.format_exc()
            self.logs.append('[Step 2 Error, Requirements Generated Failed]: ' + err_msg)
            traceback.print_exc()

    # 第二步，生成dockerfile
    def gen_dockerfile(self):
        try:
            with open(os.path.join(self.cur_dir, 'algorithm-dockerfile.template'), 'r') as f:
                templates = f.read()

            pre_command_lines = ""
            for each_command in self.pre_command:
                pre_command_lines += ("RUN " + each_command + "\n")
            requirements_lines = "RUN pip --timeout=1000000 install -r vsource_requirements.txt \n"
            templates = templates.format(pre_command_lines, requirements_lines)

            with open(os.path.join(self.service_dir, 'vsource_dockerfile'), 'w') as f:
                f.write(templates)
            self.logs.append('[Step 2/6] Dockerfile has been successfully generated!')
        except Exception as e:
            err_msg = traceback.format_exc()
            self.logs.append('[Step 2 Error, Dockerfile Generated Failed]: ' + err_msg)
            traceback.print_exc()

    # 第三步，生成vsource_configs.py
    def gen_configs(self):
        try:
            with open(os.path.join(self.cur_dir, 'algorithm-configs.template'), 'r') as f:
                templates = f.read()

            with open(os.path.join(self.service_dir, 'vsource_configs.py'), 'w') as f:
                f.write(templates)
            self.logs.append('[Step 3/6] Configfile has been successfully generated!')
        except Exception as e:
            err_msg = traceback.format_exc()
            self.logs.append('[Step 3 Error, ConfigFile Generated Failed]: ' + err_msg)
            traceback.print_exc()

    # 第四步，service.py
    def gen_service(self):
        try:
            with open(os.path.join(self.cur_dir, 'algorithm-service.template'), 'r') as f:
                templates = f.read()
            import_lines = 'import {}'.format(self.service_filename)
            param_init_lines = ""
            for each_input_param_name in self.input_params:
                param_init_lines += ('            {} = info_dict[\'{}\']\n'.format(each_input_param_name, each_input_param_name))
            function_lines =  "            out = {}.{}(".format(self.service_filename, self.service_interface)
            for each_input_param_name in list(self.input_params.keys())[:-1]:
                function_lines += (each_input_param_name + ', ')
            function_lines += (list(self.input_params.keys())[-1] + ")\n")
            output_lines = "            return out\n"
            templates = templates.format(import_lines, param_init_lines + function_lines + output_lines)
            with open(os.path.join(self.service_dir, 'vsource_service.py'), 'w') as f:
                f.write(templates)
            self.logs.append('[Step 4/6] Servicefile has been successfully generated!')
        except Exception as e:
            err_msg = traceback.format_exc()
            self.logs.append('[Step 4 Error, Servicefile Generated Failed]: ' + err_msg)
            traceback.print_exc()

    # 第五步，vsource_deploy_kube.yaml
    def gen_deployment_kube(self):
        try:
            with open(os.path.join(self.cur_dir, 'algorithm-deploy-kube.template'), 'r') as f:
                templates = f.read()
            templates = templates.format(
                self.algorithm_name, self.algorithm_name, self.algorithm_name,
                self.algorithm_name, self.algorithm_name, self.algorithm_name, self.algorithm_name,
            )
            with open(os.path.join(self.service_dir, 'vsource_deploy_kube.yaml'), 'w') as f:
                f.write(templates)
            self.logs.append('[Step 5/6] Deployment kube file has been successfully generated!')
        except Exception as e:
            err_msg = traceback.format_exc()
            self.logs.append('[Step 5 Error, Deployment kube file Generated Failed]: ' + err_msg)
            traceback.print_exc()

    # 第六步，deployment-compose.yaml
    def gen_deployment_compose(self):
        try:
            with open(os.path.join(self.cur_dir, 'algorithm-deploy-compose.template'), 'r') as f:
                templates = f.read()
            templates = templates.format(
                self.algorithm_name, self.algorithm_name, self.algorithm_name
            )
            with open(os.path.join(self.service_dir, 'vsource_deploy_compose.yaml'), 'w') as f:
                f.write(templates)
            self.logs.append('[Step 6/6] Deployment compose file has been successfully generated!')
        except Exception as e:
            err_msg = traceback.format_exc()
            self.logs.append('[Step 6 Error, Deployment compose file Generated Failed]: ' + err_msg)
            traceback.print_exc()

    def generate(self):
        self.gen_requirements_txt()
        self.gen_dockerfile()
        self.gen_configs()
        self.gen_service()
        self.gen_deployment_kube()
        self.gen_deployment_compose()

def parse_algorithm(config):
    a = AlgorithmContainerConfigParser(config)
    a.generate()

if __name__ == '__main__':
    config = {
        'algorithm-name': 'new_algorithm',
        'version': '1.0.1',
        'author': 'xcy',
        'service-dir': '/Users/ecohnoch/Desktop/VSOURCE_PLATFORM/algorithm_api/new_algorithm',
        'service-filename': 'test_function.py',
        'service-interface': 'random_number',
        'requirements': [
            'numpy==1.19.0',
            'opencv-python'
        ],
        'pre-command': [
            'apt install -y libgl1-mesa-glx'
        ],
        'input-params': {
            'lower': 'lower limit for this function',
            'upper': 'upper limit for this function'
        },
        'output-params': {
            'result': 'calculated results, as a random number'
        }
    }

    a = AlgorithmContainerConfigParser(config)
    a.generate()



# -*- coding: utf-8 -*-
# @Author   : Ecohnoch(xcy)
# @File     : __init__.py.py
# @Function : TODO

from vsource_algorithm.loginInstance import login
from vsource_algorithm.algorithms.face_recognition import face_recognition
from vsource_algorithm.algorithms.face_detection import face_detection
from vsource_algorithm.algorithms.face_attribute import face_attribute
from vsource_algorithm.algorithms.speaker_recognition import speaker_recognition

from vsource_algorithm.scripts.algorithm_scripts.algorithms import parse_algorithm




# -*- coding: utf-8 -*-
# @Author   : Ecohnoch(xcy)
# @File     : setup.py
# @Function : TODO

#!/usr/bin/env python
# coding=utf-8

from setuptools import setup, find_packages

setup(
    name='vsource_algorithm',
    version='v1.0.9',
    description=(
        'Algorithm python library linked to vsource platform.'
    ),
    long_description=open('README.rst', encoding='utf8').read(),
    author='Ecohnoch',
    author_email='542305306@qq.com',
    maintainer='Ecohnoch',
    maintainer_email='542305306@qq.com',
    license='BSD License',
    packages=find_packages(),
    platforms=["all"],
    url='https://github.com/VSOURCE-Platform/VSOURCE-Library',
    install_requires=[
        'requests'
    ],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Operating System :: OS Independent',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Programming Language :: Python :: 3.6',
        'Topic :: Software Development :: Libraries'
    ],
)


import os
import json
import time
import requests
import traceback

import vsource_algorithm.exceptions as exceptions
from vsource_algorithm.loginInstance import login_instance
import vsource_algorithm.configs as configs

def speaker_recognition(speaker_path1, speaker_path2, max_interval=configs.max_interval):
    speaker_upload_url = configs.speaker_upload_url
    speaker_submit_url = configs.speaker_submit_url
    speaker_result_url = configs.speaker_result_url
    headers = {'Cookie': login_instance.cookie}
    with open(speaker_path1, 'rb') as f1:
        filename1 = os.path.split(speaker_path1)[-1]
        ext1 = os.path.splitext(speaker_path1)[-1]
        if ext1 not in ['.wav']:
            raise exceptions.FileExtNotValid('[VSOURCE-Lib] 音频扩展名不对，请检查是否是wav格式')
        if ext1 == '.wav':
            filetype = 'audio/wav'
        uploaded_response1 = requests.post(speaker_upload_url, files={
            'file': (filename1, f1.read(), filetype)
        }, headers=headers)
        if uploaded_response1.status_code == 403:
            raise exceptions.LoginError('[VSOURCE-Lib] 登陆信息失败，请先正常的登陆')

        uploaded_path1 = json.loads(uploaded_response1.content)['return_path']
    with open(speaker_path2, 'rb') as f2:
        filename2 = os.path.split(speaker_path2)[-1]
        ext2 = os.path.splitext(speaker_path2)[-1]
        if ext2 not in ['.wav']:
            raise exceptions.FileExtNotValid('[VSOURCE-Lib] 音频扩展名不对，请检查是否是wav格式')
        if ext2 == '.wav':
            filetype = 'audio/wav'
        uploaded_response2 = requests.post(speaker_upload_url, files={
            'file': (filename2, f2.read(), filetype)
        }, headers=headers)
        if uploaded_response1.status_code == 403:
            raise exceptions.LoginError('[VSOURCE-Lib] 登陆信息失败，请先正常的登陆')
        uploaded_path2 = json.loads(uploaded_response2.content)['return_path']
    post_params = {
        'speaker_name1': uploaded_path1,
        'speaker_name2': uploaded_path2
    }
    response = requests.post(speaker_submit_url, data=post_params, headers=headers)
    if response.status_code == 403:
        raise exceptions.LoginError('[VSOURCE-Lib] 登陆信息失败，请先正常的登陆')
    response_dict = json.loads(response.content)
    task_id = response_dict['id']
    start_time = time.time()
    while True:
        # 轮询查询结果
        if time.time() - start_time >= max_interval:
            raise exceptions.TimeOutException('[VSOURCE-Lib] 请求超时，未知错误')
        face_result = requests.get(speaker_result_url, params={'id': task_id}, headers=headers, timeout=max_interval)
        face_ans = json.loads(face_result.content)
        if face_ans['status'] != 200:
            time.sleep(configs.interval)
            continue
        assert face_ans['status'] == 200
        if face_ans['result']['status'] == 'error':
            return -1
        return float(face_ans['result']['score'])

    raise exceptions.TimeOutException('[VSOURCE-Lib] 请求超时，未知错误')